import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.StringTokenizer;

public class ConvertToXML {

	public static void main(String[] args) {
		String templateXML = args[0];
		String csvFile = args[1];
		String propsFile = args[2];
		String outXMLFile = args[3];
		
		//maintainPartyTEMPLATE.xml fsk.csv configFSK.txt mpFSK.xml
		//java -cp "bin" ConvertToXML maintainPartyTEMPLATE.xml fsk.csv configFSK.txt mpFSK.xml
		//java -cp "bin" ConvertToXML maintainPartyTEMPLATE.xml alka.csv configALKA.txt mpALKA.xml

		try {
			HashMap props = parsePropertyFile(propsFile); 
			parseXMLTemplate(templateXML, csvFile, outXMLFile, props );
		} catch (Exception e) {
			e.printStackTrace();
		}
	}


	public static void parseXMLTemplate(String templateXML, String csvFile, String XMLFile, HashMap props) throws IOException {

		StringBuffer mpTemplateXML = new StringBuffer();
		mpTemplateXML = parseTemplate(templateXML);


		File rs = new File(csvFile);
		BufferedReader csvIn = new BufferedReader( new InputStreamReader( new FileInputStream(rs), "Windows-1252") );
		
		try {
			createBatchXML(csvIn, mpTemplateXML, XMLFile, props );
		} catch (Exception e) {
			System.out.println(e.getMessage());
			e.printStackTrace();
			System.exit(-1);
		}

	}
	public static void createBatchXML(BufferedReader csvIn,	StringBuffer mpTemplateXML,	String XMLFile, HashMap props) throws IOException {
		int j = 1;
		String item;
		String[] inceptionDate;
		String[] expiryDate;
		String company = "";
		//List values = new ArrayList();
		BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(XMLFile),"UTF-8"));
		bw.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");bw.newLine();
		while ((item = csvIn.readLine()) != null) {
			OutputDOT(j++);
			try {
				StringBuffer newXML_Txn = new StringBuffer(mpTemplateXML);
				String tempS1 = newXML_Txn.toString().trim();
				
				String[] csvItems = item.split("\\|");
				if( csvItems == null || csvItems.length == 0 )
					continue;
				
				processCSVLine( csvItems, newXML_Txn, props  );
			
				bw.write(newXML_Txn.toString().trim());
				String tempS2 = newXML_Txn.toString().trim();
				bw.newLine();
				
			} catch (NumberFormatException e) {
				e.printStackTrace();
			}
		}
		bw.close();
	}

	private static void OutputDOT(int i) {
		if(i==1)      System.out.print("I"); 
		if(i==10)     System.out.print("X"); 
		if(i==100)    System.out.print("C"); 
		if(i==1000)   System.out.print("M"); 
		if(i==10000)  System.out.print("XM"); 
		if(i==100000) System.out.print("X2M"); 
		
		if( i< 10) {
			 System.out.print("."); return;
		}
		if( i< 100) {
			 if( (i % 10) == 0) System.out.print("."); 
			 return;
		}
		if( i< 1000) {
			 if( (i % 100) == 0) System.out.print("."); 
			 return;
		}
		if( i< 10000) {
			 if( (i % 1000) == 0) System.out.print("."); 
			 return;
		}
		if( (i % 10000) == 0) System.out.print("."); 
	}


	private static void processCSVLine(String[] csvItems, StringBuffer newXML_Txn, HashMap props) {
		// given one line of CSV values, go through all the props and replace the AA_ with the CSV value according to the props column number
		for( int i=0; i<csvItems.length; i++ ) {
			String iS = Integer.toString(i+1);
			String templateStr = (String)props.get(iS); // 2=AAA_BD : replaceStr=AAA_BD
			if( templateStr == null || templateStr.length() == 0)
				continue;
			String csvValue = csvItems[i]; // 1990-01-01
			replaceTemplateString( templateStr, csvValue.trim(), newXML_Txn);
		}
		
		// replace <ProvinceStateType>AAA_ADST1</ProvinceStateType> with ""
		replaceString( "<ProvinceStateType>AAA_ADST1</ProvinceStateType>",  newXML_Txn );
		replaceString( "<ProvinceStateType>AAA_ADST2</ProvinceStateType>",  newXML_Txn );
		replaceString( "<ProvinceStateType>AAA_ADST3</ProvinceStateType>",  newXML_Txn );
		// filter out any lines containing AAA
		filterString( "AAA",  newXML_Txn );
		
	}

	private static void filterString(String strAAA, StringBuffer newXML_Txn) {
		StringBuffer filteredLine = new StringBuffer();
		String[] lines = newXML_Txn.toString().split("\n");
		for( int i=0; i< lines.length; i++) {
			String line = lines[i];
			if( line.indexOf(strAAA) < 0 )
				filteredLine.append(line+"\n");
		}
		newXML_Txn.delete(0, newXML_Txn.length()-1);
		newXML_Txn.append(filteredLine);
	}


	private static void replaceString(String string, StringBuffer newXML_Txn) {
		int st = newXML_Txn.indexOf(string);
		if( st  < 0 ) 
			return;
		int end = st + string.length();
		newXML_Txn = newXML_Txn.replace(st, end, "");
	}


	private static void replaceTemplateString(String templateStr, String csvValue, StringBuffer newXML_Txn) {
		// templateStr = AAA_BD
		// csvValue = 1990-01-01
		// templateStr AAA_BD:DATE=DD.MM.YYYY
		// templateStr AAA_AT:FIXED=1
		
		if( csvValue == null || csvValue.length() == 0 )
			return;
		
		csvValue = makeXMLSafe(csvValue);
		
		// handle date conversions
		String[] templateArray = templateStr.split(":");
		if( templateArray.length == 2) {
			templateStr = templateArray[0];
			// convert csvValue according to date format
			csvValue = formatConvert( csvValue, templateArray[1] );
		}
		
		int st = newXML_Txn.indexOf(templateStr);
		if( st  < 0 ) 
			return;
		int end = st + templateStr.length();
		newXML_Txn = newXML_Txn.replace(st, end, csvValue);
		
	}

	/**
	 * There are five XML Special characters which needs to be escaped :
	 *     
	    & - &amp;
	    < - &lt;
	    > - &gt;
	    " - &quot;
	    ' - &apos;
	 */
	private static String makeXMLSafe(String csvValue) {
		// replace special XML characters : & @ < > 
		// replace special XML characters : & @ < > gt, lt, quot, amp, apos
		/*
    double quotes ( " ) are escaped to &quot;
    ampersand ( & ) is escaped to &amp;
    single quotes ( ' ) are escaped to &apos;
    less than ( < ) is escaped to &lt;
    greater than ( > ) is escaped to &gt;		 * 
		 */
		//return StringEscapeUtils.escapeXml(csvValue);
		csvValue = replaceStrings(csvValue, "&", "&amp;"); // must be first
		csvValue = replaceStrings(csvValue, "\"", "&quot;");
		csvValue = replaceStrings(csvValue, "'", "&apos;");
		csvValue = replaceStrings(csvValue, "<", "&lt;");
		csvValue = replaceStrings(csvValue, ">", "&gt;");
		return csvValue;
	}
	
	private static String replaceStrings(String s, String from, String to) {
		StringBuffer sb = new StringBuffer(s);
		int start = 0,i=0;
		while( (i = sb.indexOf(from, start)) >= 0 ) {
			sb.replace(i, i+from.length(), to);
			start = i+1;
		}
		return sb.toString();
	}


	private static String formatConvert(String csvValue, String csvFormat) {
			String[] formats = csvFormat.split("\\,");
			
			switch (formats[0]) {
            case "DATE":  
            	return dateConvert(csvValue, formats[1]);
            case "FIXED":  
            	return formats[1];
			}
			return csvValue;				
	}

	// csvValue = 1990-01-01
	// templateStr DD.MM.YYYY

	private static String dateConvert(String csvValue, String csvFormat) {
		// convert cssValue to YYYY-MM-YY eg 1933-11-15
		try {
			String year  = getYear(  csvValue, csvFormat);
			String month = getMonth( csvValue, csvFormat);
			String day   = getDay(   csvValue, csvFormat);
			return year+"-"+month+"-"+day;
		} catch (Exception e) {
			return csvValue;
		}
	}


	// return the DD fields
	private static String getDay(String csvValue, String csvFormat) {
		int st = csvFormat.indexOf("D");
		return csvValue.substring(st, st+2);
	}


	private static String getMonth(String csvValue, String csvFormat) {
		int st = csvFormat.indexOf("M");
		return csvValue.substring(st, st+2);
	}


	private static String getYear(String csvValue, String csvFormat) {
		int st = csvFormat.indexOf("Y");
		return csvValue.substring(st, st+4);
	}





	final static int RAU_FILE_SIZE = 21970;
	final static int D_FILE_SIZE = 752;

	public static StringBuffer parseTemplate(String RAUXMLTemplate)
			throws IOException {
		File fil = new File(".");
		String pathName = fil.getAbsolutePath();
		/** windows* */
		// String path = pathName + "\\" + RAUXMLTemplate;
		/** unix* */
		String path = pathName + "/" + RAUXMLTemplate;
		// System.out.println("path%%%"+path);
		FileReader fr = new FileReader(path);
		BufferedReader br = new BufferedReader(fr);
		char[] update_xml = new char[RAU_FILE_SIZE];
		StringBuffer update_xml_buf = new StringBuffer();
		br.read(update_xml, 0, RAU_FILE_SIZE);
		update_xml_buf.append(update_xml);
		// System.out.println(update_xml_buf.toString());
		return update_xml_buf;
	}

	public static HashMap parsePropertyFile(String FileName) throws IOException {
		File fil = new File(".");
		String pathName = fil.getAbsolutePath();
		/** windows* */
		// String path = pathName + "\\" + FileName;
		/** unix* */
		String path = pathName + "/" + FileName;
		HashMap values = new HashMap();
		FileReader rs = new FileReader(path);
		BufferedReader in = new BufferedReader(rs);
		String item;
		while ((item = in.readLine()) != null) {
			StringTokenizer st1 = new StringTokenizer(item, "=");
			while (st1.hasMoreTokens()) {
				String key = st1.nextToken();
				String value = st1.nextToken();
				// System.out.println("value property"+value);
				values.put(key, value);
			}

		}
		return values;
	}


}
